var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var _a;
import { readdir, writeFile, mkdir, readFile } from "fs/promises";
import { Command } from "commander";
import inquirer from "inquirer";
const EDITOR = process.env["EDITOR"];
const BLOGS_DIR = process.cwd() + ((_a = process.env["BLOGS_DIR"]) !== null && _a !== void 0 ? _a : "/blogs");
export function app() {
    const program = new Command();
    const blog_app = program.command("blog");
    blog_app.command("create").action(() => __awaiter(this, void 0, void 0, function* () {
        yield create_blog();
    }));
    blog_app
        .command("edit", "command to edit blog")
        .argument("<id>")
        .action((id) => __awaiter(this, void 0, void 0, function* () {
        yield edit_blog(id);
    }));
    blog_app.parse();
}
function create_blog() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!EDITOR) {
            process.env["EDITOR"] = "nvim";
        }
        const user_blog_meta = yield inquirer.prompt([
            {
                name: "name",
                message: "What is the title of your blog?",
            },
            {
                name: "description",
                message: "Enter a short description for your blog",
            },
            {
                type: "editor",
                name: "content",
                message: "Please enter your blog now.",
                postfix: ".md",
            },
        ]);
        const created_at = Date.now();
        const updated_at = created_at;
        const blog_id = yield get_new_blog_id();
        const { name, description, content } = user_blog_meta;
        const new_blog_meta = {
            name,
            description,
            id: blog_id,
            created_at,
            updated_at,
        };
        yield save_blog(new_blog_meta, content);
    });
}
function edit_blog(blog_id) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!EDITOR) {
            process.env["EDITOR"] = "nvim";
        }
        const blog_path = `${BLOGS_DIR}/blog_${blog_id}`;
        const [blog_content, blog_meta] = yield Promise.all([
            get_blog(blog_path),
            get_blog_meta(blog_path),
        ]);
        const { content } = yield inquirer.prompt([
            {
                type: "editor",
                name: "content",
                default: blog_content,
                message: "Please enter your blog now.",
                postfix: ".md",
            },
        ]);
        const new_blog_meta = Object.assign(Object.assign({}, blog_meta), { updated_at: Date.now() });
        yield save_blog(new_blog_meta, content);
    });
}
function get_new_blog_id() {
    return __awaiter(this, void 0, void 0, function* () {
        const blog_count = (yield readdir(BLOGS_DIR, { withFileTypes: true })).filter((dirent) => dirent.isDirectory()).length;
        return blog_count + 1;
    });
}
function save_blog(blog_meta, content) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = `${BLOGS_DIR}/blog_${blog_meta.id}`;
        yield mkdir(path, { recursive: true });
        yield Promise.all([
            write_meta_data(blog_meta, path),
            write_blog(content, path),
        ]);
    });
}
function get_blog(path) {
    return __awaiter(this, void 0, void 0, function* () {
        return (yield readFile(path + "/blog.md", "utf8")).toString();
    });
}
function get_blog_meta(path) {
    return __awaiter(this, void 0, void 0, function* () {
        const meta = JSON.parse((yield readFile(path + "/meta.json", "utf8")).toString());
        return meta;
    });
}
function write_meta_data(blog_meta, path) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = JSON.stringify(blog_meta);
        yield writeFile(path + "/meta.json", data);
    });
}
function write_blog(data, path) {
    return __awaiter(this, void 0, void 0, function* () {
        yield writeFile(path + "/blog.md", data);
    });
}
